//
//  AppDelegate.m
//  CameraKitDemo
//
//  Created by Kevin on 2015/6/24.
//  Copyright (c) 2015 IPEVO Inc. All rights reserved.
//

#import "AppDelegate.h"

#define USE_LIVE_STREAM_LAYER_FOR_VIDEO 1 // 0: display video from CIImage manually

@interface AppDelegate ()

@property (weak) IBOutlet NSWindow *window;
@property (nonatomic, strong) ICCamera * currentCamera;
@property (strong) CIImage * latestImage;
#if USE_LIVE_STREAM_LAYER_FOR_VIDEO == 0
@property (nonatomic, strong) CALayer * videoLayer;
#endif
@property (nonatomic, strong) NSMutableSet * observers;
@property (nonatomic, strong) ICLiveStreamLayer * liveStreamLayer;
@end

@implementation AppDelegate

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification {
    // Insert code here to initialize your application
    self.observers = [NSMutableSet set];
    NSNotificationCenter * nc = [NSNotificationCenter defaultCenter];
    __weak AppDelegate * weakSelf = self;
    [self.observers addObject:[nc addObserverForName:ICNotificationCameraAttached object:nil queue:nil usingBlock:^(NSNotification *note) {
        [weakSelf.camerasArrayController addObject:note.userInfo[ICCamerasManagerNotificationKeyCamera]];
    }]];
    [self.observers addObject:[nc addObserverForName:ICNotificationCameraDetached object:nil queue:nil usingBlock:^(NSNotification *note) {
        [weakSelf.camerasArrayController removeObject:note.userInfo[ICCamerasManagerNotificationKeyCamera]];
    }]];
    [self.observers addObject:[nc addObserverForName:ICNotificationCameraPropertyDidChange object:nil queue:nil usingBlock:^(NSNotification *note) {
            if (note.userInfo[ICCameraPropertyKeyFocus]) {
                weakSelf.focusSlider.integerValue = [note.userInfo[ICCameraPropertyKeyFocus] integerValue];
            }
            if (note.userInfo[ICCameraPropertyKeyAutoFocus]) {
                BOOL mf = ![note.userInfo[ICCameraPropertyKeyAutoFocus] boolValue];
                weakSelf.mfButton.state = (mf) ? NSOnState : NSOffState;
            }
            if (note.userInfo[ICCameraPropertyKeyAutoExposure]) {
                BOOL aeLock = ![note.userInfo[ICCameraPropertyKeyAutoExposure] boolValue];
                weakSelf.aeLockButton.state = (aeLock) ? NSOnState : NSOffState;
            }
            if (note.userInfo[ICCameraPropertyKeyAutoWhiteBalance]) {
                BOOL awb = [note.userInfo[ICCameraPropertyKeyAutoWhiteBalance] boolValue];
                weakSelf.awbButton.state = (awb) ? NSOnState : NSOffState;
            }
            if (note.userInfo[ICCameraPropertyKeyExposure]) {
                weakSelf.exposureSlider.integerValue = [note.userInfo[ICCameraPropertyKeyExposure] integerValue];
            }
            if (note.userInfo[ICCameraPropertyKeyWhiteBalance]) {
                NSInteger value = [note.userInfo[ICCameraPropertyKeyWhiteBalance] integerValue];
                weakSelf.wbSlider.integerValue = value;
            }
    }]];
    [self.observers addObject:[nc addObserverForName:ICNotificationIPEVOCameraCommand object:nil queue:nil usingBlock:^(NSNotification *note) {
        if (note.object == self.currentCamera) {
            if ([note.userInfo[@"command"] isEqualToString:ICIPEVOCameraCommandSnapshot]) {
                // save self.latestImage to file
            } else if ([note.userInfo[@"command"] isEqualToString:ICIPEVOCameraCommandFocusBegin]) {
                weakSelf.videoContainerView.layer.borderColor = [NSColor redColor].CGColor;
            } else if ([note.userInfo[@"command"] isEqualToString:ICIPEVOCameraCommandFocusEnd]) {
                weakSelf.videoContainerView.layer.borderColor = [NSColor greenColor].CGColor;
                dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(2 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                    weakSelf.videoContainerView.layer.borderColor = [NSColor clearColor].CGColor;
                });
            } else if ([note.userInfo[@"command"] isEqualToString:ICIPEVOCameraCommandFocusFailed]) {
                weakSelf.videoContainerView.layer.borderColor = [NSColor clearColor].CGColor;
            } else if ([note.userInfo[@"command"] isEqualToString:ICIPEVOCameraCommandSetToSingleFocus]) {
                NSLog(@"Set to sigle focus");
            } else if ([note.userInfo[@"command"] isEqualToString:ICIPEVOCameraCommandSetToContinuousFocus]) {
                NSLog(@"Set to continuous focus");
            } else if ([note.userInfo[@"command"] isEqualToString:ICIPEVOCameraCommandReportExposureValue]) {
            } else if ([note.userInfo[@"command"] isEqualToString:ICIPEVOCameraCommandReportAutoExposureValue]) {
            } else if ([note.userInfo[@"command"] isEqualToString:ICIPEVOCameraCommandReportCurrentDevice]) {
            } else if ([note.userInfo[@"command"] isEqualToString:ICIPEVOCameraCommandFocusAgain]) {
                [weakSelf focus:nil];
            }
        }
    }]];
    
    [[ICCamerasManager sharedManager] startMonitoring];
    
    [self.camerasArrayController addObserver:self forKeyPath:@"selectionIndex" options:NSKeyValueObservingOptionNew context:nil];
    [self.formatsArrayController addObserver:self forKeyPath:@"selectionIndex" options:NSKeyValueObservingOptionNew context:nil];

    [self.videoContainerView setWantsLayer:YES];
    self.videoContainerView.layer.backgroundColor = [NSColor blackColor].CGColor;
    self.videoContainerView.layer.borderWidth = 4;
    self.videoContainerView.layer.borderColor = [NSColor clearColor].CGColor;
    
#if USE_LIVE_STREAM_LAYER_FOR_VIDEO == 0
    self.videoLayer = [CALayer layer];
    self.videoLayer.delegate = self;
    self.videoLayer.frame = self.videoContainerView.layer.bounds;
    self.videoLayer.autoresizingMask = NSViewWidthSizable | NSViewHeightSizable;
    self.videoLayer.contentsGravity = kCAGravityResizeAspect;
    self.videoLayer.actions = @{@"contents": [NSNull null]};
    [self.videoContainerView.layer addSublayer:self.videoLayer];
#endif
}

- (void)applicationWillTerminate:(NSNotification *)aNotification {
    // Insert code here to tear down your application
}

- (void)setCurrentCamera:(ICCamera *)currentCamera
{
    [[ICCameraStreamProxy sharedProxy] removeStreamObserver:self forCamera:_currentCamera];
    _currentCamera = currentCamera;
    [[ICCameraStreamProxy sharedProxy] addStreamObserver:self forCamera:_currentCamera];
    
#if USE_LIVE_STREAM_LAYER_FOR_VIDEO == 1
    ICCaptureSession * captureSession = [[ICCameraStreamProxy sharedProxy] captureSessionForCamera:_currentCamera];
    [self.liveStreamLayer removeFromSuperlayer];
    self.liveStreamLayer = [[ICLiveStreamLayer alloc] initWithSession:captureSession];
    self.liveStreamLayer.autoresizingMask = NSViewHeightSizable | NSViewWidthSizable;
    self.liveStreamLayer.frame = self.videoContainerView.layer.bounds;
    [self.videoContainerView.layer addSublayer:self.liveStreamLayer];
#endif
}

- (void)displayLayer:(CALayer *)layer
{
    NSBitmapImageRep* rep = [[NSBitmapImageRep alloc] initWithCIImage:self.latestImage];
    layer.contents = (__bridge id)(rep.CGImage);
}

- (void) setupUI
{
    SInt16 value;
    ICCamera * camera = self.currentCamera;
    
    if ([camera isKindOfClass:[ICUVCCamera class]]) {
        [self.imageAdjustButton setEnabled:YES];
        if ([camera getBrightness:&value ofType:ICCameraPropertyValueMinimum] == ICCameraCommandSucceeded) {
            self.brightnessSlider.minValue = value;
        }
        if ([camera getBrightness:&value ofType:ICCameraPropertyValueMaximum] == ICCameraCommandSucceeded) {
            self.brightnessSlider.maxValue = value;
        }
        if ([camera getBrightness:&value ofType:ICCameraPropertyValueCurrent] == ICCameraCommandSucceeded) {
            self.brightnessSlider.integerValue = value;
        }
        if ([camera getContrast:&value ofType:ICCameraPropertyValueMinimum] == ICCameraCommandSucceeded) {
            self.contrastSlider.minValue = value;
        }
        if ([camera getContrast:&value ofType:ICCameraPropertyValueMaximum] == ICCameraCommandSucceeded) {
            self.contrastSlider.maxValue = value;
        }
        if ([camera getContrast:&value ofType:ICCameraPropertyValueCurrent] == ICCameraCommandSucceeded) {
            self.contrastSlider.integerValue = value;
        }
        if ([camera getGamma:&value ofType:ICCameraPropertyValueMinimum] == ICCameraCommandSucceeded) {
            self.gammaSlider.minValue = value;
        }
        if ([camera getGamma:&value ofType:ICCameraPropertyValueMaximum] == ICCameraCommandSucceeded) {
            self.gammaSlider.maxValue = value;
        }
        if ([camera getGamma:&value ofType:ICCameraPropertyValueCurrent] == ICCameraCommandSucceeded) {
            self.gammaSlider.integerValue = value;
        }
        if ([camera getHue:&value ofType:ICCameraPropertyValueMinimum] == ICCameraCommandSucceeded) {
            self.hueSlider.minValue = value;
        }
        if ([camera getHue:&value ofType:ICCameraPropertyValueMaximum] == ICCameraCommandSucceeded) {
            self.hueSlider.maxValue = value;
        }
        if ([camera getHue:&value ofType:ICCameraPropertyValueCurrent] == ICCameraCommandSucceeded) {
            self.hueSlider.integerValue = value;
        }
        if ([camera getSaturation:&value ofType:ICCameraPropertyValueMinimum] == ICCameraCommandSucceeded) {
            self.saturationSlider.minValue = value;
        }
        if ([camera getSaturation:&value ofType:ICCameraPropertyValueMaximum] == ICCameraCommandSucceeded) {
            self.saturationSlider.maxValue = value;
        }
        if ([camera getSaturation:&value ofType:ICCameraPropertyValueCurrent] == ICCameraCommandSucceeded) {
            self.saturationSlider.integerValue = value;
        }
        if ([camera getSharpness:&value ofType:ICCameraPropertyValueMinimum] == ICCameraCommandSucceeded) {
            self.sharpnessSlider.minValue = value;
        }
        if ([camera getSharpness:&value ofType:ICCameraPropertyValueMaximum] == ICCameraCommandSucceeded) {
            self.sharpnessSlider.maxValue = value;
        }
        if ([camera getSharpness:&value ofType:ICCameraPropertyValueCurrent] == ICCameraCommandSucceeded) {
            self.sharpnessSlider.integerValue = value;
        }
    } else {
        [self.imageAdjustButton setEnabled:NO];
    }
    
    if (camera.capabilities & ICCameraCapabilityExposure) {
        [self.exposureButton setEnabled:YES];
        if (camera.capabilities & ICCameraCapabilityAutoExposureLock) {
            [self.aeLockButton setEnabled:YES];
        } else {
            [self.aeLockButton setEnabled:NO];
        }
        SInt16 min, max, cur;
        [self.currentCamera getExposure:&min ofType:ICCameraPropertyValueMinimum];
        [self.currentCamera getExposure:&max ofType:ICCameraPropertyValueMaximum];
        [self.currentCamera getExposure:&cur ofType:ICCameraPropertyValueCurrent];
        self.exposureSlider.minValue = min;
        self.exposureSlider.maxValue = max;
        self.exposureSlider.integerValue = cur;
        self.exposureSlider.numberOfTickMarks = max - min + 1;
        self.exposureSlider.allowsTickMarkValuesOnly = YES;
    } else {
        [self.exposureButton setEnabled:NO];
    }
    
    if (camera.capabilities & ICCameraCapabilityWhiteBalance) {
        [self.whitebalanceButton setEnabled:YES];
    } else {
        [self.whitebalanceButton setEnabled:NO];
    }
    
    if (camera.capabilities & ICCameraCapabilityManualFocus) {
        
        SInt16 min, max;
        SInt8 autoFocus = 0;
        if ([self.currentCamera getFocus:&min ofType:ICCameraPropertyValueMinimum] == ICCameraCommandSucceeded &&
            [self.currentCamera getFocus:&max ofType:ICCameraPropertyValueMaximum] == ICCameraCommandSucceeded &&
            [self.currentCamera getAutoFocus:&autoFocus] == ICCameraCommandSucceeded) {
            self.focusSlider.minValue = min;
            self.focusSlider.maxValue = max;
            self.mfButton.state = (autoFocus == 1) ? NSOffState : NSOnState;
            [self.manualFocusButton setEnabled:YES];
        } else {
            [self.manualFocusButton setEnabled:NO];
        }
    } else {
        [self.manualFocusButton setEnabled:NO];
    }
    
    if ([camera isKindOfClass:[ICIPEVOCamera class]]) {
        [self.focusButton setEnabled:YES];
    } else {
        [self.focusButton setEnabled:NO];
    }
}

- (IBAction)openImageAdjustView:(id)sender {
    [self.imageAdjustPopover showRelativeToRect:[sender bounds] ofView:sender preferredEdge:NSMinYEdge];
}

- (IBAction)openExposureView:(id)sender {
    [self.exposurePopover showRelativeToRect:[sender bounds] ofView:sender preferredEdge:NSMinYEdge];
}

- (IBAction)openWhitebalanceView:(id)sender {
    [self.whitebalancePopover showRelativeToRect:[sender bounds] ofView:sender preferredEdge:NSMinYEdge];
}

- (IBAction)openManualFocusView:(id)sender {
    [self.manualFocusPopover showRelativeToRect:[sender bounds] ofView:sender preferredEdge:NSMinYEdge];
}

- (IBAction)setBrightness:(id)sender {
    [self.currentCamera setBrightness:[sender integerValue]];
}

- (IBAction)setContrast:(id)sender {
    [self.currentCamera setContrast:[sender integerValue]];
}

- (IBAction)setGamma:(id)sender {
    [self.currentCamera setGamma:[sender integerValue]];
}

- (IBAction)setHue:(id)sender {
    [self.currentCamera setHue:[sender integerValue]];
}

- (IBAction)setSaturation:(id)sender {
    [self.currentCamera setSaturation:[sender integerValue]];
}

- (IBAction)setSharpness:(id)sender {
    [self.currentCamera setSharpness:[sender integerValue]];
}

- (IBAction)resetImageParametersToDefault:(id)sender {
    SInt16 value;
    ICCamera * camera = self.currentCamera;

    if ([camera getBrightness:&value ofType:ICCameraPropertyValueDefault] == ICCameraCommandSucceeded) {
        [camera setBrightness:value];
        self.brightnessSlider.integerValue = value;
    }

    if ([camera getContrast:&value ofType:ICCameraPropertyValueDefault] == ICCameraCommandSucceeded) {
        [camera setContrast:value];
        self.contrastSlider.integerValue = value;
    }
    if ([camera getGamma:&value ofType:ICCameraPropertyValueDefault] == ICCameraCommandSucceeded) {
        [camera setGamma:value];
        self.gammaSlider.integerValue = value;
    }
    if ([camera getHue:&value ofType:ICCameraPropertyValueDefault] == ICCameraCommandSucceeded) {
        [camera setHue:value];
        self.hueSlider.integerValue = value;
    }
    if ([camera getSaturation:&value ofType:ICCameraPropertyValueDefault] == ICCameraCommandSucceeded) {
        [camera setSaturation:value];
        self.saturationSlider.integerValue = value;
    }
    if ([camera getSharpness:&value ofType:ICCameraPropertyValueDefault] == ICCameraCommandSucceeded) {
        [camera setSharpness:value];
        self.sharpnessSlider.integerValue = value;
    }
}

- (IBAction)toggleAELock:(id)sender {
    BOOL aeLock = [sender state] == NSOnState;
    [self.currentCamera setAutoExposure:!aeLock];
}

- (IBAction)setExposure:(id)sender {
    [self.currentCamera setExposure:[sender integerValue]];
}

- (IBAction)setExposureToDefault:(id)sender {
    SInt16 defaultValue;
    if ([self.currentCamera getExposure:&defaultValue ofType:ICCameraPropertyValueDefault] == ICCameraCommandSucceeded) {
        [self.currentCamera setExposure:defaultValue];
        self.exposureSlider.integerValue = defaultValue;
    }
}

- (IBAction)toggleAWB:(id)sender {
    BOOL awb = [sender state] == NSOnState;

    if (awb == NO) {
        [self.currentCamera setWhiteBalance:self.wbSlider.integerValue];
    }
    [self.currentCamera setAutoWhiteBalance:awb];
}

- (IBAction)setWhitebalance:(id)sender {
    SInt16 wbValue = [sender integerValue];
    [self.currentCamera setAutoWhiteBalance:NO];
    [self.currentCamera setWhiteBalance:wbValue];
}

- (IBAction)toggleAFMF:(id)sender {
    BOOL manualFocus = [sender state] == NSOnState;
    [self.currentCamera setAutoFocus:!manualFocus];
    if (manualFocus == NO) {
        [self.currentCamera focus];
    }
}

- (IBAction)setFocus:(id)sender {
    SInt8 inAutoFocus = 0;
    if ([self.currentCamera getAutoFocus:&inAutoFocus] == ICCameraCommandSucceeded) {
        if (inAutoFocus == 1) {
            if ([self.currentCamera setAutoFocus:0] == ICCameraCommandSucceeded) {
                inAutoFocus = 0;
            }
        }
        if (inAutoFocus == 0) {
            SInt16 min, max;
            if ([self.currentCamera getFocus:&min ofType:ICCameraPropertyValueMinimum] == ICCameraCommandSucceeded &&
                [self.currentCamera getFocus:&max ofType:ICCameraPropertyValueMaximum] == ICCameraCommandSucceeded) {

                [self.currentCamera setFocus:[sender integerValue]];
            }
        }
    }
}

- (IBAction)focus:(id)sender
{
    [self.currentCamera setAutoFocus:1];
    [self.currentCamera focus];
}

#pragma mark - KVO

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
    if (object == self.camerasArrayController) {
        if ([keyPath isEqualToString:@"selectionIndex"]) {
            dispatch_async(dispatch_get_main_queue(), ^{
                self.currentCamera = self.camerasArrayController.selectedObjects.lastObject;
                [self setupUI];
                [self.formatsArrayController removeObjects:self.formatsArrayController.arrangedObjects];
                NSArray * supportedResolutions = _currentCamera.supportedResolutions;
                [self.formatsArrayController addObjects:supportedResolutions];
                
                if (supportedResolutions.count > 0) {
                    [self.currentCamera setResolution:supportedResolutions[0]];
                    NSSize size = NSSizeFromString(supportedResolutions[0][@"resolution"]);
                    self.liveStreamLayer.contentHeightWidthRatio = size.height / size.width;
                }
            });
        }
    } else if (object == self.formatsArrayController) {
        if ([keyPath isEqualToString:@"selectionIndex"]) {
            NSDictionary * resolution = self.formatsArrayController.selectedObjects.lastObject;
            [self.currentCamera setResolution:resolution];
            NSSize size = NSSizeFromString(resolution[@"resolution"]);
            self.liveStreamLayer.contentHeightWidthRatio = size.height / size.width;
        }
    }
}

#pragma mark - ICCameraStreamProxyDelegate
- (void) cameraStreamProxy:(ICCameraStreamProxy *) cameraStreamProxy didReceiveFrame:(CIImage *) image withInfo:(NSDictionary *) info fromCamera:(ICCamera *) camera
{
    self.latestImage = image;
#if USE_LIVE_STREAM_LAYER_FOR_VIDEO == 0
    [self.videoLayer performSelectorOnMainThread:@selector(setNeedsDisplay) withObject:nil waitUntilDone:NO];
#endif
}
@end
