//
//  ICCamera.h
//  IPEVOCameraKit
//
//  Created by Kevin on 13/10/4.
//  Copyright (c) 2013 IPEVO Inc. All rights reserved.
//

#define ICNotificationCameraSetToSingleFocus	@"CameraSetToSingleFocus"
#define ICNotificationCameraSetToContinueFocus	@"CameraSetToContinueFocus"
#define ICNotificationCameraDidPressSnapshot    @"CameraDidPressSnapshot"
#define ICNotificationCameraDidBeginAutoFocus   @"CameraDidBeginAutoFocus"
#define ICNotificationCameraDidEndAutoFocus     @"CameraDidEndAutoFocus"
#define ICNotificationCameraDidCancelAutoFocus  @"CameraDidCancelAutoFocus"
#define ICNotificationCameraCustomButton        @"CameraCustomButton"
#define ICNotificationCameraPropertyDidChange   @"CameraPropertyDidChange"

#define ICCameraCapabilityResolution        0x01
#define ICCameraCapabilityExposure          0x02
#define ICCameraCapabilityAutoExposureLock  0x04
#define ICCameraCapabilityWhiteBalance      0x08
#define ICCameraCapabilityManualFocus       0x10

#define ICCameraPropertyKeyBrightness       @"brightness"
#define ICCameraPropertyKeyContrast         @"contrast"
#define ICCameraPropertyKeyGamma            @"gamma"
#define ICCameraPropertyKeyHue              @"hue"
#define ICCameraPropertyKeySaturation       @"saturation"
#define ICCameraPropertyKeySharpness        @"sharpness"
#define ICCameraPropertyKeyExposure         @"exposure"
#define ICCameraPropertyKeyFocus            @"focus"
#define ICCameraPropertyKeyAutoFocus        @"autofocus"
#define ICCameraPropertyKeyResolution       @"resolution"
#define ICCameraPropertyKeyFrequency        @"frequency"
#define ICCameraPropertyKeyAutoWhiteBalance @"autoWhiteBalance"
#define ICCameraPropertyKeyWhiteBalance     @"whiteBalance"
#define ICCameraPropertyKeyAutoExposure     @"autoExposure"

typedef enum {
    ICCameraAutoFocusModeUnknown,
    ICCameraAutoFocusModeOnce,
    ICCameraAutoFocusModeContinuous
} ICCameraAutoFocusMode;

typedef enum {
    ICCameraCommandUnsupported = -1,
    ICCameraCommandSucceeded = 0,
    ICCameraCommandFailed = 1
} ICCameraCommandReturnValue;

typedef enum {
    ICCameraPropertyValueMaximum,
    ICCameraPropertyValueMinimum,
    ICCameraPropertyValueDefault,
    ICCameraPropertyValueCurrent
} ICCameraPropertyValueType;

typedef enum {
    ICCameraPowerlineFrequencyDisabled = 0,
    ICCameraPowerlineFrequency50Hz = 1,
    ICCameraPowerlineFrequency60Hz
} ICCameraPowerlineFrequency;

@interface ICCamera : NSObject {
}

@property (nonatomic, readonly, unsafe_unretained) NSString * model;
@property (nonatomic, readonly, unsafe_unretained) NSString * shortModel;
@property (nonatomic, strong) NSString * instanceName;
@property (nonatomic, unsafe_unretained) BOOL active;
@property (nonatomic, assign) NSUInteger capabilities;
@property (nonatomic, readonly, strong) NSArray * supportedResolutions;
@property (atomic, unsafe_unretained) BOOL ready;
@property (nonatomic, readonly, strong) NSString * uuid;

+ (id) camera;

- (ICCameraCommandReturnValue) focus;

/** brightness */
- (ICCameraCommandReturnValue) setBrightness:(SInt16) value;
- (ICCameraCommandReturnValue) getBrightness:(SInt16 *) value ofType:(ICCameraPropertyValueType) type;

/** contrast */
- (ICCameraCommandReturnValue) setContrast:(SInt16) value;
- (ICCameraCommandReturnValue) getContrast:(SInt16 *) value ofType:(ICCameraPropertyValueType) type;

/** gamma */
- (ICCameraCommandReturnValue) setGamma:(SInt16) value;
- (ICCameraCommandReturnValue) getGamma:(SInt16 *) value ofType:(ICCameraPropertyValueType) type;

/** hue */
- (ICCameraCommandReturnValue) setHue:(SInt16) value;
- (ICCameraCommandReturnValue) getHue:(SInt16 *) value ofType:(ICCameraPropertyValueType) type;

/** saturation */
- (ICCameraCommandReturnValue) setSaturation:(SInt16) value;
- (ICCameraCommandReturnValue) getSaturation:(SInt16 *) value ofType:(ICCameraPropertyValueType) type;

/** sharpness */
- (ICCameraCommandReturnValue) setSharpness:(SInt16) value;
- (ICCameraCommandReturnValue) getSharpness:(SInt16 *) value ofType:(ICCameraPropertyValueType) type;

/** autoWhiteBalance */
- (ICCameraCommandReturnValue) setWhiteBalance:(SInt16) value;
- (ICCameraCommandReturnValue) getWhiteBalance:(SInt16 *) value ofType:(ICCameraPropertyValueType) type;

/** autoWhiteBalance */
- (ICCameraCommandReturnValue) setAutoWhiteBalance:(SInt8) value;
- (ICCameraCommandReturnValue) getAutoWhiteBalance:(SInt8 *) value;

/** autoFocus */
- (ICCameraCommandReturnValue) setFocus:(SInt16) value;
- (ICCameraCommandReturnValue) getFocus:(SInt16 *) value ofType:(ICCameraPropertyValueType) type;

/** autoFocus */
- (ICCameraCommandReturnValue) setAutoFocus:(SInt8) value;
- (ICCameraCommandReturnValue) getAutoFocus:(SInt8 *) value;
- (ICCameraCommandReturnValue) getAutoFocusMode:(ICCameraAutoFocusMode *) value;

/** AutoExposure: true-enable;  false-disable*/
- (ICCameraCommandReturnValue) setAutoExposure:(BOOL) value;  // UVC
- (ICCameraCommandReturnValue) getAutoExposure:(BOOL *) value;

/** set the related exposure value */
- (ICCameraCommandReturnValue) setExposure:(SInt16) value;
- (ICCameraCommandReturnValue) getExposure:(SInt16 *) value ofType:(ICCameraPropertyValueType) type;

/** set Iris indeed, **ONLY** controlable when AutoExposure disabled 曝光值 */
- (ICCameraCommandReturnValue) setIris:(SInt16) value;
- (ICCameraCommandReturnValue) getIris:(SInt16 *) value ofType:(ICCameraPropertyValueType) type;

/** frequency: 0-auto(disabled); 1-50Hz, 2-60Hz */
- (ICCameraCommandReturnValue) setFrequency:(ICCameraPowerlineFrequency) value;
- (ICCameraCommandReturnValue) getFrequency:(ICCameraPowerlineFrequency *) value ofType:(ICCameraPropertyValueType) type;

- (void) prepareToWork;
- (void) prepareToDetatch;

- (void) updateSelfInfo;

- (ICCameraCommandReturnValue) setResolution:(NSDictionary *) resolutionInfo;

- (void) setUserObject:(id) object forKey:(NSString *) key;
- (id) getUserObjectForKey:(NSString *) key;
@end