//
//  ICCameraWireless.h
//  IPEVOCameraKit
//
//  Created by Kevin on 13/10/4.
//  Copyright (c) 2013 IPEVO Inc. All rights reserved.
//

#import "ICCamera.h"

#define ICNotificationCameraSettingAvailable @"CameraSettingAvailable"

typedef enum IPV_HID_COMMAND {
    SET_POWER_FREQUENCY = 0x01, // 0x00 → 60Hz, 0x01 → 50Hz
    SET_AUTO_EXPOSURE   = 0x02, // 0x00 → On, 0x01 → Off
    SET_EXPOSURE_VALUE  = 0x03, // 0x01 ~ 0x0f
    SET_FOCUS           = 0x04,
    GET_FOCUS_MODE      = 0x05, // 0x00 → AF-S, 0x01 → AF-C
    GET_EXPOSURE_VALUE  = 0x06, // 0x01 ~ 0x0f
    GET_POWER_FREQUENCY = 0x07, // 0x00 → 60Hz, 0x01 → 50Hz
    GET_CUR_DEVICE      = 0x08, // 0x04 → P2V, 0x05 → Ziggi, 0x06 → Ziggi HD, 0x07 → VZ-1, 0x08 → P2V-R
    GET_FOCUS_STATUS    = 0x09, // 0x00 → not focusing, 0x01 → focusing
    GET_ALL             = 0x0A,
    GET_AUTO_EXPOSURE   = 0x0B, // 0x00 → On, 0x01 → Off
    GET_API_VERSION     = 0x0C
} IPV_HID_COMMAND;

@interface ICCameraWireless : ICCamera <NSNetServiceDelegate>
@property (nonatomic, strong) NSNetService * deviceNetService;
@property (nonatomic, strong) NSNetService * streamNetService;
@property (nonatomic, strong) NSNetService * camSettingNetService;
@property (nonatomic, strong) NSNetService * devSettingNetService;
@property (nonatomic, readonly, unsafe_unretained) BOOL controllable;

- (IPEVOCameraAFSubMode) afSubMode;
- (void) getSupportedResolutionsWithUsername:(NSString *) username andPassword:(NSString *) password completionHandler:(void (^)(NSArray * resolutions)) completionHandler failureHandler:(void (^)(NSError * error)) failureHandler;
- (void) getCurrentResolutionWithUsername:(NSString *) username andPassword:(NSString *) password completionHandler:(void (^)(NSString * resolution)) completionHandler failureHandler:(void (^)(NSError * error)) failureHandler;
- (void) setResolution:(NSString *) resolution withUsername:(NSString *) username andPassword:(NSString *) password completionHandler:(void (^)(NSString * resolution)) completionHandler failureHandler:(void (^)(NSError * error)) failureHandler;

- (NSURL *) urlForWebConfig;
- (void) checkUsername:(NSString *) username andPassword:(NSString *) password successHandler:(void (^)(void)) successHandler failureHandler:(void (^)(NSError * error)) failureHandler;

- (NSInteger) currentDevice;
@end