//
//  ICUVCCamera.h
//  IPEVOCameraKit
//
//  Created by Kevin on 13/10/4.
//  Copyright (c) 2013 IPEVO Inc. All rights reserved.
//

#import "ICCamera.h"
#import "ICCameraAVFoundationCapable.h"
#import <IOKit/usb/IOUSBLib.h>
#import <IOKit/IOKitLib.h>
#import <IOKit/IOMessage.h>
#import <IOKit/IOCFPlugIn.h>
#import <IOKit/usb/IOUSBLib.h>
#import <IOKit/hid/IOHIDLib.h>
#import <IOKit/hid/IOHIDKeys.h>
#import <IOKit/hid/IOHIDUsageTables.h>
#import <IOKit/hidsystem/IOHIDLib.h>
#import <IOKit/hidsystem/IOHIDShared.h>
#import <IOKit/hidsystem/IOHIDParameter.h>

/* Video Class-Specific Request Codes */
#define SET_CUR                         0x01
#define GET_CUR                         0x81
#define GET_MIN                         0x82
#define GET_MAX                         0x83
#define GET_RES                         0x84
#define GET_LEN                         0x85
#define GET_INFO                        0x86
#define GET_DEF                         0x87

/* Video Interface Subclass Codes */
#define SC_UNDEFINED                    0x00
#define SC_VIDEOCONTROL                 0x01
#define SC_VIDEOSTREAMING               0x02
#define SC_VIDEO_INTERFACE_COLLECTION   0x03

/* VideoStreaming interface controls select */
#define VS_CONTROL_UNDEFINED            0x00
#define VS_ICOBE_CONTROL                0x01
#define VS_COMMIT_CONTROL               0x02
#define VS_STILL_ICOBE_CONTROL          0x03
#define VS_STILL_COMMIT_CONTROL         0x04
#define VS_STILL_IMAGE_TRIGGER_CONTROL  0x05
#define VS_STREAM_ERROR_CODE_CONTROL    0x06
#define VS_GENERATE_KEY_FRAME_CONTROL   0x07
#define VS_UPDATE_FRAME_SEGMENT_CONTROL 0x08
#define VS_SYNC_DELAY_CONTROL           0x09

/* VideoControl interface controls */
#define VC_CONTROL_UNDEFINED            0x00
#define VC_VIDEO_POWER_MODE_CONTROL     0x01
#define VC_REQUEST_ERROR_CODE_CONTROL   0x02

/* USB Terminal Types */
#define TT_VENDOR_SPECIFIC              0x0100
#define TT_STREAMING                    0x0101

/* Terminal controls */
#define TE_CONTROL_UNDEFINED            0x00

/* Selector Unit controls */
#define SU_CONTROL_UNDEFINED            0x00
#define SU_INPUT_SELECT_CONTROL         0x01

/* Camera Terminal controls */
#define CT_CONTROL_UNDEFINED            		0x00
#define CT_SCANNING_MODE_CONTROL        		0x01
#define CT_AE_MODE_CONTROL              		0x02
#define CT_AE_ICIORITY_CONTROL          		0x03
#define CT_EXPOSURE_TIME_ABSOLUTE_CONTROL       0x04
#define CT_EXPOSURE_TIME_RELATIVE_CONTROL       0x05
#define CT_FOCUS_ABSOLUTE_CONTROL       		0x06
#define CT_FOCUS_RELATIVE_CONTROL       		0x07
#define CT_FOCUS_AUTO_CONTROL           		0x08
#define CT_IRIS_ABSOLUTE_CONTROL        		0x09
#define CT_IRIS_RELATIVE_CONTROL        		0x0a
#define CT_ZOOM_ABSOLUTE_CONTROL        		0x0b
#define CT_ZOOM_RELATIVE_CONTROL        		0x0c
#define CT_PANTILT_ABSOLUTE_CONTROL     		0x0d
#define CT_PANTILT_RELATIVE_CONTROL     		0x0e
#define CT_ROLL_ABSOLUTE_CONTROL        		0x0f
#define CT_ROLL_RELATIVE_CONTROL        		0x10
#define CT_ICIVACY_CONTROL              		0x11

/* Processing Unit controls */
#define PU_CONTROL_UNDEFINED                        0x00
#define PU_BACKLIGHT_COMPENSATION_CONTROL           0x01
#define PU_BRIGHTNESS_CONTROL                       0x02
#define PU_CONTRAST_CONTROL                         0x03
#define PU_GAIN_CONTROL                             0x04
#define PU_POWER_LINE_FREQUENCY_CONTROL             0x05
#define PU_HUE_CONTROL                              0x06
#define PU_SATURATION_CONTROL                       0x07
#define PU_SHARPNESS_CONTROL                        0x08
#define PU_GAMMA_CONTROL                            0x09
#define PU_WHITE_BALANCE_TEMPERATURE_CONTROL        0x0a
#define PU_WHITE_BALANCE_TEMPERATURE_AUTO_CONTROL   0x0b
#define PU_WHITE_BALANCE_COMPONENT_CONTROL          0x0c
#define PU_WHITE_BALANCE_COMPONENT_AUTO_CONTROL     0x0d
#define PU_DIGITAL_MULTIPLIER_CONTROL               0x0e
#define PU_DIGITAL_MULTIPLIER_LIMIT_CONTROL         0x0f
#define PU_HUE_AUTO_CONTROL                         0x10
#define PU_ANALOG_VIDEO_STANDARD_CONTROL            0x11
#define PU_ANALOG_LOCK_STATUS_CONTROL               0x12

/* VideoStreaming interface controls */
#define VS_CONTROL_UNDEFINED            0x00
#define VS_ICOBE_CONTROL                0x01
#define VS_COMMIT_CONTROL               0x02
#define VS_STILL_ICOBE_CONTROL          0x03
#define VS_STILL_COMMIT_CONTROL         0x04
#define VS_STILL_IMAGE_TRIGGER_CONTROL  0x05
#define VS_STREAM_ERROR_CODE_CONTROL    0x06
#define VS_GENERATE_KEY_FRAME_CONTROL   0x07
#define VS_UPDATE_FRAME_SEGMENT_CONTROL 0x08
#define VS_SYNC_DELAY_CONTROL           0x09

/* External Terminal types */
#define EXTERNAL_VENDOR_SPECIFIC        0x0400
#define COMPOSITE_CONNECTOR             0x0401
#define SVIDEO_CONNECTOR                0x0402
#define COMPONENT_CONNECTOR             0x0403

@interface ICUVCCamera : ICCameraAVFoundationCapable

@property (nonatomic, unsafe_unretained) UInt16 vendorId;
@property (nonatomic, unsafe_unretained) UInt16 productId;
@property (nonatomic, unsafe_unretained) UInt32 locationId;
@property (nonatomic, unsafe_unretained) UInt16 versionNumber;
@property (nonatomic, unsafe_unretained) IOUSBInterfaceInterface300 ** streamControlInterface;
@property (nonatomic, strong) NSString * captureDeviceLocationId;
@property (nonatomic, unsafe_unretained) NSUInteger videoControlInterfaceNumber;
@property (nonatomic, copy)   NSString * productString;

- (BOOL) sendUVCInterfaceCommand:(UInt8) cmd toControlSelector:(UInt8) cs ofInterfaceSubclass:(UInt8) unit withData:(void *) data andSize:(UInt16) size;
- (BOOL) sendUVCControlCommand:(NSInteger) cmd index:(NSInteger) idx data:(void *) data;

- (void) prepareToWorkWithDevice:(io_object_t) device;

- (BOOL) setupVideoControlInterface:(io_object_t) device;

@end
